import requests
import logging

class Client:
    def __init__(self, p2_store_endpoint):
        self.log = logging.getLogger(__name__)
        self.p2_store_endpoint = p2_store_endpoint

    def list_experiments(self):
        url = self.p2_store_endpoint+'/experiments'
        try:
            resp = requests.get( url, timeout=5)
            return resp.json() if (resp.status_code == 200) else None
        except requests.exceptions.RequestException as e:
            self.log.error("Error getting all experiments")
            return None

    def get_experiment(self, filepath):
        url = self.p2_store_endpoint+'/experiment/'+filepath
        try:
            resp = requests.get( url, timeout=5)
            return resp.json() if (resp.status_code == 200) else None
        except requests.exceptions.RequestException as e:
            self.log.error("Error getting experiment {}: {}".format(filepath, e))
            return None


    def add_experiment(self, filename, data):
        url = self.p2_store_endpoint+'/experiment/'+filename
        try:
            resp = requests.post(
                url,
                json=data,
                timeout=5)
            return resp.json() if (resp.status_code == 201) else None
        except requests.exceptions.RequestException as e:
            self.log.error("Error creating experiment {}".format(e))
            return None

    def delete_experiment(self, filename):
        url = self.p2_store_endpoint+'/experiment/'+filename
        resp = requests.delete( url, timeout=5)
        return resp.json() if (resp.status_code == 200) else None


    def create_session(self, exp_id, session):
        url = self.p2_store_endpoint+'/exp_session/'+exp_id
        try:
            resp = requests.post(
                url, 
                json=session,
                timeout=5)
            return exp_id if (resp.status_code == 201) else None
        except requests.exceptions.RequestException as e:
            self.log.error("Error creating session: {}".format(e))
            return None

    def get_version(self):
        try:
            res = requests.get(self.p2_store_endpoint+'/version', timeout=10)
            if (res.status_code != 200):
                self.log.warning("No version found")
                return None
            else:
                return res.json()
        except requests.exceptions.RequestException as e:
            self.log.error("Error getting p2store version {}".format(e))
            return None


    def get_sessions(self, filter: dict):
        try:
            res = requests.get(self.p2_store_endpoint+'/exp_session', 
                params=filter,
                timeout=10
            )
            if (res.status_code != 200):
                self.log.warning("No session found")
                return None
            else:
                sessions = res.json()
                if (len(sessions) is 0):
                    self.log.warning("No session found")
                return sessions
        except requests.exceptions.RequestException as e:
            self.log.error("Error getting sessions: {}".format(e))
            return None
    
    def get_exp_log(self, exp_id, log_type):
        try:
            url = self.p2_store_endpoint+'/exp_log/'+exp_id+'/'+log_type
            res = requests.get(url, timeout=5)
            return res.json()
        except requests.exceptions.RequestException as e:
            self.log.error("Error getting session {} log: {}".format(exp_id, e))
            return None

    def update_session( self, exp_id, attrs ):
        try:
            res = requests.put(self.p2_store_endpoint+"/exp_session/{}".format(exp_id), json=attrs)
            return exp_id if res.status_code == 200 else None
        except requests.exceptions.RequestException as e:
            self.log.error("Error updating session {}: {}".format(exp_id, e))
            return None

    def update_log( self, exp_id, log ):
        try: 
            res = requests.post(self.p2_store_endpoint+"/exp_log/{}".format(exp_id), json=log)
            return exp_id if res.status_code == 201 else None
        except requests.exceptions.RequestException as e:
            self.log.error("Error updating log {}: {}".format(exp_id, e))
            return None
